
set echo on

CREATE OR REPLACE PACKAGE BODY trades_load AS

   ------------------------------------------------------------

   FUNCTION trades_transform( p_source_data IN SYS_REFCURSOR )
      RETURN transaction_ntt
      PIPELINED 
      PARALLEL_ENABLE (PARTITION p_source_data BY ANY) IS

      /* Array of input record type... */
      TYPE aat_source_data IS TABLE OF trades_staging%ROWTYPE
         INDEX BY PLS_INTEGER;
      aa_source_data aat_source_data;
      r_source_data  trades_staging%ROWTYPE;

      /* Output record types... */
      r_trade         trade_ot         := trade_ot();
      r_trade_leg     trade_leg_ot     := trade_leg_ot();
      r_leg_valuation leg_valuation_ot := leg_valuation_ot();

   BEGIN

      LOOP

         FETCH p_source_data BULK COLLECT INTO aa_source_data LIMIT 100;

         FOR i IN 1 .. aa_source_data.COUNT LOOP

            /* Work with a single record... */
            r_source_data := aa_source_data(i);

            /* Reset the variables... */
            r_trade         := trade_ot();
            r_trade_leg     := trade_leg_ot();
            r_leg_valuation := leg_valuation_ot();

            /* Prepare and pipe the trade record... */
            IF r_source_data.leg_no = 1 THEN

               r_trade.transaction_id  := r_source_data.trade_id;
               r_trade.product_type    := r_source_data.product_type;
               r_trade.counterparty    := r_source_data.counterparty;
               r_trade.trading_book    := r_source_data.trading_book;
               r_trade.trade_timestamp := r_source_data.trade_timestamp;
               r_trade.maturity_date   := r_source_data.maturity_date;

               PIPE ROW (r_trade);

            END IF;

            /* Prepare and pipe the trade_leg record... */
            r_trade_leg.transaction_id := r_source_data.trade_id;
            r_trade_leg.leg_no         := r_source_data.leg_no;
            r_trade_leg.trade_amount   := r_source_data.trade_amount;
            r_trade_leg.currency       := r_source_data.currency;
            r_trade_leg.trade_price    := r_source_data.trade_price;

            PIPE ROW (r_trade_leg);

            /* Prepare and pipe the leg_valuation record... */
            r_leg_valuation.transaction_id := r_source_data.trade_id;
            r_leg_valuation.leg_no         := r_source_data.leg_no;
            r_leg_valuation.valuation_date := r_source_data.valuation_date;
            r_leg_valuation.market_value   := r_source_data.market_value;

            PIPE ROW (r_leg_valuation);

         END LOOP;

         EXIT WHEN p_source_data%NOTFOUND;

      END LOOP;
      CLOSE p_source_data;

      RETURN;

   END trades_transform;

   ------------------------------------------------------------

   PROCEDURE load_trades IS
   BEGIN

      INSERT FIRST
         WHEN record_type = 'TRADES'
         THEN
            INTO trades ( trade_id
                        , product_type
                        , counterparty
                        , trade_timestamp
                        , trading_book
                        , maturity_date
                        )
            VALUES ( trade_id 
                   , product_type
                   , counterparty
                   , trade_timestamp
                   , trading_book
                   , maturity_date
                   )
         WHEN record_type = 'TRADE_LEGS'
         THEN
            INTO trade_legs ( trade_id
                            , leg_no
                            , trade_amount
                            , currency
                            , trade_price
                            )
            VALUES ( trade_id
                   , leg_no
                   , trade_amount
                   , currency
                   , trade_price
                   )
         WHEN record_type = 'LEG_VALUATIONS'
         THEN
            INTO leg_valuations ( trade_id
                                , leg_no
                                , valuation_date
                                , market_value
                                )
            VALUES ( trade_id
                   , leg_no
                   , valuation_date
                   , market_value
                   )
      SELECT ilv.record_type
      ,      COALESCE(
                ilv.trade_rec.transaction_id,
                ilv.trade_leg_rec.transaction_id,
                ilv.leg_valuation_rec.transaction_id
                )                                    AS trade_id
      ,      COALESCE(
                ilv.trade_leg_rec.leg_no,
                ilv.leg_valuation_rec.leg_no
                )                                    AS leg_no
      ,      ilv.trade_rec.product_type              AS product_type
      ,      ilv.trade_rec.counterparty              AS counterparty
      ,      ilv.trade_rec.trade_timestamp           AS trade_timestamp
      ,      ilv.trade_rec.trading_book              AS trading_book
      ,      ilv.trade_rec.maturity_date             AS maturity_date
      ,      ilv.trade_leg_rec.trade_amount          AS trade_amount
      ,      ilv.trade_leg_rec.currency              AS currency
      ,      ilv.trade_leg_rec.trade_price           AS trade_price 
      ,      ilv.leg_valuation_rec.valuation_date    AS valuation_date
      ,      ilv.leg_valuation_rec.market_value      AS market_value
      FROM (
            SELECT CASE
                      WHEN VALUE(nt) IS OF TYPE (trade_ot)
                      THEN 'TRADES'
                      WHEN VALUE(nt) IS OF TYPE (trade_leg_ot)
                      THEN 'TRADE_LEGS'
                      ELSE 'LEG_VALUATIONS'
                   END AS record_type
            ,      TREAT(VALUE(nt) AS trade_ot)         AS trade_rec
            ,      TREAT(VALUE(nt) AS trade_leg_ot)     AS trade_leg_rec
            ,      TREAT(VALUE(nt) AS leg_valuation_ot) AS leg_valuation_rec
            FROM   TABLE(
                      trades_load.trades_transform(
                         CURSOR( SELECT * FROM trades_staging ) ) ) nt
           ) ilv;

      DBMS_OUTPUT.PUT_LINE( SQL%ROWCOUNT || ' rows inserted.' );

   END load_trades;

END trades_load;
/


